library(phylobase)
library(ggplot2)
library(phangorn)
library(distory)
library(reshape2)
library(gridExtra)

#############   init vars   #####################

basepath      <- "C:/devwork/partitionfinder/Project_PFanalysis/"

# expected tree filename format: "RAxML_bipartitions.<partitionscheme>.final"
 datasets      <- list("Anderson_2013", "Bergsten_2013", "Broughton_2013", "Brown_2012", "Caterino_2001", "Cognato_2001", 
                      "Day_2013", "Devitt_2013", "Dornburg_2012", "Dsouli_2011", "Ekrem_2010", "Elias_2009", 
                      "Fishbein_2001", "Fong_2012", "Grande_2013", "Guschanski_2013", "Kang_2013a", "Kang_2013b", "Kaffenberger_2011", 
                      "Kawahara_2013", "Lartillot_2012", "Leavitt_2013", "Li_2008", "Murray_2013", "Rightmyer_2013", 
                      "Sauquet_2011", "Seago_2011", "Sharanowski_2011", "Siler_2013", "Tolley_2013", "Unmack_2013", "Wainwright_2012", 
                      "Ward_2010", "Welton_2013"
                      )     # list your dataset folder names here!
# datasets      <- list("Brown_2012", "Caterino_2001", "Fong_2012", "Seago_2011")
pfschemes     <- c("PFbic","PFaicc", "PFall","PFnone")            # Partitioning scheme names. These must be in your tree filenames.
fsuffix       <- "final"                                       # this is the end part of the tree filename.





# 1. examine the topological distance between PFbic trees and the trees from the other 3 schemes
#     -- see if distances are significantly different depending on which of the 3 other schemes is used.
analysis_topo_distances <- function(df)
{
  print("TOPOLOGICAL DISTANCE COMPARISONS")
  
  distPD.aicc =  df[df$PF.scheme=='PFaicc', 'dist.PD']  # i.e Path Distance between BIC and AICc tree
  distPD.all =  df[df$PF.scheme=='PFall', 'dist.PD']    # i.e Path Distance between BIC and ALL tree
  distPD.none =  df[df$PF.scheme=='PFnone', 'dist.PD']  # i.e Path Distance between BIC and NONE tree
  print( paste('PATH DISTANCES for PFbic vs PFaicc compared to PFbic vs PFall, P-value = ', suppressWarnings( wilcox.test(distPD.aicc, distPD.all)[[3]])) )
  print( paste('PATH DISTANCES for PFbic vs PFaicc compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distPD.aicc, distPD.none)[[3]])) )
  print( paste('PATH DISTANCES for PFbic vs PFall compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distPD.all, distPD.none)[[3]])) )
  
  distRF.aicc =  df[df$PF.scheme=='PFaicc', 'dist.RF']
  distRF.all =  df[df$PF.scheme=='PFall', 'dist.RF']
  distRF.none =  df[df$PF.scheme=='PFnone', 'dist.RF']
  print( paste('ROBINSON FOULDS DISTANCES for PFbic vs PFaicc compared to PFbic vs PFall, P-value = ', suppressWarnings( wilcox.test(distRF.aicc, distRF.all)[[3]])) )
  print( paste('ROBINSON FOULDS DISTANCES for PFbic vs PFaicc compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distRF.aicc, distRF.none)[[3]])) )
  print( paste('ROBINSON FOULDS DISTANCES for PFbic vs PFall compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distRF.all, distRF.none)[[3]])) )
  
  distMS.aicc =  df[df$PF.scheme=='PFaicc', 'dist.MS']
  distMS.all =  df[df$PF.scheme=='PFall', 'dist.MS']
  distMS.none =  df[df$PF.scheme=='PFnone', 'dist.MS']
  print( paste('MATCHING SPLIT DISTANCES for PFbic vs PFaicc compared to PFbic vs PFall, P-value = ', suppressWarnings( wilcox.test(distMS.aicc, distMS.all)[[3]])) )
  print( paste('MATCHING SPLIT DISTANCES for PFbic vs PFaicc compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distMS.aicc, distMS.none)[[3]])) )
  print( paste('MATCHING SPLIT DISTANCES for PFbic vs PFall compared to PFbic vs PFnone, P-value = ', suppressWarnings( wilcox.test(distMS.all, distMS.none)[[3]])) )
  print("======================================================")
  
  print("MEAN TOPOLOGICAL DISTANCES vs BIC FOR ALL TREES ")
  print( paste("PFbic vs PFaicc: (RF, MS, PD): ", mean(distRF.aicc), mean(distMS.aicc), mean(distPD.aicc), sep=" ") )
  print( paste("PFbic vs PFall (RF, MS, PD): ", mean(distRF.all), mean(distMS.all), mean(distPD.all), sep=" ") )
  print( paste("PFbic vs PFnone (RF, MS, PD): ", mean(distRF.none), mean(distMS.none), mean(distPD.none), sep=" ") )
  print("======================================================")
  
  print("MEAN TOPOLOGICAL DISTANCES vs BIC FOR TREES WITH DISTANCE > 0 ")  
  print( paste("PFbic vs PFaicc: (RF, MS, PD): ", mean(distRF.aicc[distRF.aicc>0]), mean(distMS.aicc[distMS.aicc>0]), mean(distPD.aicc[distPD.aicc>0]), sep=" ") )
  print( paste("PFbic vs PFall (RF, MS, PD): ", mean(distRF.all[distRF.all>0]), mean(distMS.all[distMS.all>0]), mean(distPD.all[distPD.all>0]), sep=" ") )
  print( paste("PFbic vs PFnone (RF, MS, PD): ", mean(distRF.none[distRF.none>0]), mean(distMS.none[distMS.none>0]), mean(distPD.none[distPD.none>0]), sep=" ") )
  print("======================================================")
}

# 2. examine the number of conflicted nodes per dataset
#    -- see how many nodes are conflicted vs BIC per PF scheme
analysis_numconflicts <- function(df)
{
  BS <- extract_BSs(df)
  print( table(subset(BS, subset=type=='diff')$PF.scheme) )
  #    -- get percentage of conflicted nodes of each non-BIC tree versus the corresponding BIC tree
  df.nonBIC <- df[which(df$PF.scheme!='PFbic'),]
  percentDiff <- mapply(FUN=function(x,y){ length( unlist( strsplit(x,",") ) ) / y },x=df.nonBIC$Diffs.nodes, y=df.nonBIC$nodes.total)
  names(percentDiff) <- df.nonBIC$PF.scheme
  percentDiff <- data.frame(scheme=names(percentDiff), percdiff=percentDiff)
  print("PERCENTAGE OF NODES THAT ARE DIFFERENT TO THE BIC TREE")
  print("======================================================")
  print(percentDiff)

  #    -- plot with 4 panels: percentage of diff nodes by scheme; 3 metrics of topological distance by scheme
  fig1a <- ggplot(percentDiff) + geom_boxplot(aes(x=scheme, y=percdiff)) + labs(y='proportion conflicted', x='') + scale_x_discrete(labels=c(expression(Opt[AICc]), expression(Adhoc[feat]), expression(Adhoc[one]))) + theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)))
  fig1b <- ggplot(df.nonBIC) + geom_boxplot(aes(y=dist.RF, x=PF.scheme)) + labs(y='RF distance', x='') + scale_x_discrete(labels=c(expression(Opt[AICc]), expression(Adhoc[feat]), expression(Adhoc[one]))) + theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)))
  fig1c <- ggplot(df.nonBIC) + geom_boxplot(aes(y=dist.MS, x=PF.scheme)) + labs(y='MS distance', x='') + scale_x_discrete(labels=c(expression(Opt[AICc]), expression(Adhoc[feat]), expression(Adhoc[one]))) + theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)))
  fig1d <- ggplot(df.nonBIC) + geom_boxplot(aes(y=dist.PD, x=PF.scheme)) + labs(y='PD distance', x='') + scale_x_discrete(labels=c(expression(Opt[AICc]), expression(Adhoc[feat]), expression(Adhoc[one]))) + theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)))
  grid.arrange(fig1a, fig1b, fig1c, fig1d, ncol=2)
  return(percentDiff)
}

# 3. Examine mean branch lengths and bootstrap support by scheme and test for significant changes
# - the second arg is used in the call to significance_deltabl_deltabs(). It is the proportion of all branches to use for testing changes to distribution of BL.
analysis_meanBLBS <- function(df, proportion=1.0)
{
  
  # extract the actual numerical lists of BLs
  df.bl_lists <- dcast(data = df, formula = dataset ~ PF.scheme, value.var = 'BL.all' )
  # gives us a numeric matrix like this:
  #   rownames       PFbic      PFaicc    PFall     PFnone
  #   Brown_2012     listofBLs listofBLs listofBLs  listofBLs
  #   Caterino_2001  listofBLs listofBLs listofBLs  listofBLs
  #   Fong_2012      listofBLs listofBLs listofBLs  listofBLs
  #   Seago_2011     listofBLs listofBLs listofBLs  listofBLs
  #   ...
  
  # add fields with the mean of only the longest 'proportion' of branches. 
  #   e.g. get the mean of the longest 0.25 of all branches. 
  #   e.g. get the mean of the longest 1.0  of all branches, which is mean of ALL branches 
  df.bl <- df.bl_lists
  df.bl <- cbind( df.bl, BLbic.mean.longest=apply(df.bl_lists, 1, function(row){
    BL <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BLs
    longestmean <- mean(BL[1:floor(length(BL)*proportion)])                     # BLmean for PFbic
    return(longestmean)
  }) )
  df.bl <- cbind( df.bl, BLaicc.mean.longest=apply(df.bl_lists, 1, function(row){
    BL <- sort(as.double( unlist(strsplit(row['PFaicc'], ",")) ), decreasing=T)   # sorted list of BLs
    longestmean <- mean(BL[1:floor(length(BL)*proportion)])                    # BLmean for PFaicc
    return(longestmean)
  }) )
  df.bl <- cbind( df.bl, BLall.mean.longest=apply(df.bl_lists, 1, function(row){
    BL <- sort(as.double( unlist(strsplit(row['PFall'], ",")) ), decreasing=T)   # sorted list of BLs
    longestmean <- mean(BL[1:floor(length(BL)*proportion)])                    # BLmean for PFall
    return(longestmean)
  }) )
  df.bl <- cbind( df.bl, BLnone.mean.longest=apply(df.bl_lists, 1, function(row){
    BL <- sort(as.double( unlist(strsplit(row['PFnone'], ",")) ), decreasing=T)   # sorted list of BLs
    longestmean <- mean(BL[1:floor(length(BL)*proportion)])                    # BLmean for PFnone
    return(longestmean)
  }) )
  print(df.bl)
  
  bic_v_aicc <- df.bl$BLbic.mean.longest - df.bl$BLaicc.mean.longest
  bic_v_all <- df.bl$BLbic.mean.longest - df.bl$BLall.mean.longest
  bic_v_none <- df.bl$BLbic.mean.longest - df.bl$BLnone.mean.longest
  
  #    - run sign test on the differences between the means to see if there is a directional bias on mean BL or BS due to PF scheme
    print("BINOM TESTS FOR SIGNIFICANT OVERALL EFFECT ON MEAN BL DUE TO PSCHEME")
    print( binom.test(length(which(bic_v_aicc < 0)), n=length(which(bic_v_aicc!=0)), p=0.5, alternative="two.sided") )
    print( binom.test(length(which(bic_v_all < 0)), n=length(which(bic_v_all!=0)), p=0.5, alternative="two.sided") )
    print( binom.test(length(which(bic_v_none < 0)), n=length(which(bic_v_none!=0)), p=0.5, alternative="two.sided") )
    print("=============================================================================")
    print("MW TESTS FOR SIGNIFICANT OVERALL EFFECT ON MEAN BL DUE TO PSCHEME")
    print( wilcox.test(df.bl$BLbic.mean.longest, df.bl$BLaicc.mean.longest, alternative="two.sided", paired=T) )
    print( wilcox.test(df.bl$BLbic.mean.longest, df.bl$BLall.mean.longest, alternative="two.sided", paired=T) )
    print( wilcox.test(df.bl$BLbic.mean.longest, df.bl$BLnone.mean.longest, alternative="two.sided", paired=T) )
    print("=============================================================================")
    #print("BINOM TESTS FOR SIGNIFICANT OVERALL EFFECT ON MEAN BS DUE TO PSCHEME")
    #print( binom.test(length(which(df.delta$bs.bic_v_aicc < 0)), n=length(which(df.delta$bs.bic_v_aicc!=0)), p=0.5, alternative="two.sided") )
    #print( binom.test(length(which(df.delta$bs.bic_v_all < 0)), n=length(which(df.delta$bs.bic_v_all!=0)), p=0.5, alternative="two.sided") )
    #print( binom.test(length(which(df.delta$bs.bic_v_none < 0)), n=length(which(df.delta$bs.bic_v_none!=0)), p=0.5, alternative="two.sided") )
    #print("=============================================================================")
  
  print("OVERALL MEAN BL BY SCHEME: ")
  cat( "PFbic: ", mean(df.bl$BLbic.mean.longest), "\n" )
  cat( "PFaicc: ", mean(df.bl$BLaicc.mean.longest), "\n" )
  cat( "PFall: ", mean(df.bl$BLall.mean.longest), "\n" )
  cat( "PFnone: ", mean(df.bl$BLnone.mean.longest), "\n" )
  print("=============================================================================")
  print("OVERALL MEAN BS BY SCHEME: ")
  print( aggregate(df$BS.mean, by=list(df$PF.scheme), FUN=mean) )
  print("=============================================================================")
  print("")
  
  #    - panel with parallel coords of all datasets
  BLmean <- melt(df.bl, value.name = "value", variable.name = "PF.scheme", id.vars = c('dataset'), measure.vars = c('BLall.mean.longest','BLaicc.mean.longest','BLbic.mean.longest','BLnone.mean.longest') )
  fig2a <- ggplot(BLmean, aes(x = PF.scheme, y = value, group=dataset)) + 
    geom_line(alpha=0.5, size=0.25) + 
    geom_point(size=2, alpha=0.5) + 
    scale_x_discrete("", labels=c(expression(Adhoc[feat]), expression(Opt[AICc]),  expression(Opt[BIC]), expression(Adhoc[one]))) + 
    scale_y_continuous("Mean branch length") +
    theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)) )
  fig2a
  
  # extract the mean BSs
  df.bs <- dcast(data = df, formula = dataset ~ PF.scheme, value.var = 'BS.mean' )
  BSmean <- melt(df.bs, value.name = "value", variable.name = "PF.scheme", id.vars = c('dataset'), measure.vars = c('PFall','PFaicc','PFbic','PFnone') )
  fig2b <- ggplot(BSmean, aes(x = PF.scheme, y = value, group=dataset)) + 
    geom_line(alpha=0.5, size=0.25) + 
    geom_point(size=2, alpha=0.5) + 
    scale_x_discrete("", labels=c(expression(Adhoc[feat]), expression(Opt[AICc]),  expression(Opt[BIC]), expression(Adhoc[one]))) + 
    scale_y_continuous("Mean bootstrap support") +
    theme(axis.title.y = element_text(size = rel(0.8)), axis.text.y = element_text(size = rel(0.6)) )
  
  grid.arrange(fig2a, fig2b, ncol=1)
  figout <- arrangeGrob(fig2a, fig2b)
  #ggsave("Fig2.pdf", figout, width = 6.75, units = "in", height = 5.5)


#   # add fields containing difference between mean of each PF scheme and PFbic
#   df.bs <- cbind( df.bs, bs.bic_v_aicc=apply(df.bs, 1, function(row){ 
#     return( (row[1]-row[2]) ) 
#   }) )
#   df.bs <- cbind( df.bs, bs.bic_v_all=apply(df.bs, 1, function(row){ 
#     return( (row[1]-row[3]) ) 
#   }) )
#   df.bs <- cbind( df.bs, bs.bic_v_none=apply(df.bs, 1, function(row){ 
#     return( (row[1]-row[4]) ) 
#   }) )
#   print(df.bs)
}

analysis_distributionBLBS <- function(df, proportion)
{
  # get lists of BLs per dataset/scheme
  
  df.bl_lists <- dcast(data = df, formula = dataset ~ PF.scheme, value.var = 'BL.all' )
  df.mwbl <- data.frame( dataset=df.bl_lists$dataset, Pbl.bic_v_aicc=apply(df.bl_lists, 1, function(row){
    BLbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BLs
    BLbic <- BLbic[1:floor(length(BLbic)*proportion)]                         # take the top 'proportion' of the BLs for MW test
    BLaicc <- sort(as.double( unlist(strsplit(row['PFaicc'], ",")) ), decreasing=T)
    BLaicc <- BLaicc[1:floor(length(BLaicc)*proportion)]  
    p <- wilcox.test(BLbic, BLaicc, alternative = "two.sided")$p.value
    return(p)
  }))
  df.mwbl <- cbind( df.mwbl, Pbl.bic_v_all=apply(df.bl_lists, 1, function(row){
    BLbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BLs
    BLbic <- BLbic[1:floor(length(BLbic)*proportion)]       
    BLall <- sort(as.double( unlist(strsplit(row['PFall'], ",")) ), decreasing=T)
    BLall <- BLall[1:floor(length(BLall)*proportion)]
    p <- wilcox.test(BLbic, BLall, alternative = "two.sided")$p.value
    return(p)
  }))
  df.mwbl <- cbind( df.mwbl, Pbl.bic_v_none=apply(df.bl_lists, 1, function(row){
    BLbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BLs
    BLbic <- BLbic[1:floor(length(BLbic)*proportion)]       
    BLnone <- sort(as.double( unlist(strsplit(row['PFnone'], ",")) ), decreasing=T)
    BLnone <- BLnone[1:floor(length(BLnone)*proportion)]  
    p <- wilcox.test(BLbic, BLnone, alternative = "two.sided")$p.value
    return(p)
  }))
  
  # Examine change in mean BL and mean BS as well as change in the full set (i.e. distribution) of BLs and BSs per tree, due to scheme
  #    - get dataframe with deltas and also P-values wrt the sets of BLs and BSs 

  print("DATASETS WITH SIGNIFICANTLY (p < 0.05) CHANGED DISTRIBUTION OF BLs COMPARED TO THE BIC TREE: ")
  print( "bic - aicc:")
  print( df.mwbl[which(df.mwbl$Pbl.bic_v_aicc<0.05), 'dataset'])
  print( "bic - all:")
  print( df.mwbl[which(df.mwbl$Pbl.bic_v_all<0.05), 'dataset'])
  print( "bic - none:")
  print( df.mwbl[which(df.mwbl$Pbl.bic_v_none<0.05), 'dataset'])
  print("=============================================================================")
  
  df.bs_lists <- dcast(data = df, formula = dataset ~ PF.scheme, value.var = 'BS.all' )
  df.mwbs <- data.frame( dataset=df.bs_lists$dataset, Pbs.bic_v_aicc=apply(df.bs_lists, 1, function(row){
    BSbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BSs
    BSbic <- BSbic[1:floor(length(BSbic)*proportion)]                         # take the top 'proportion' of the BSs for MW test
    BSaicc <- sort(as.double( unlist(strsplit(row['PFaicc'], ",")) ), decreasing=T)
    BSaicc <- BSaicc[1:floor(length(BSaicc)*proportion)]  
    p <- wilcox.test(BSbic, BSaicc, alternative = "two.sided")$p.value
    return(p)
  }))
  df.mwbs <- cbind( df.mwbs, Pbs.bic_v_all=apply(df.bs_lists, 1, function(row){
    BSbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BSs
    BSbic <- BSbic[1:floor(length(BSbic)*proportion)]       
    BSall <- sort(as.double( unlist(strsplit(row['PFall'], ",")) ), decreasing=T)
    BSall <- BSall[1:floor(length(BSall)*proportion)]
    p <- wilcox.test(BSbic, BSall, alternative = "two.sided")$p.value
    return(p)
  }))
  df.mwbs <- cbind( df.mwbs, Pbs.bic_v_none=apply(df.bs_lists, 1, function(row){
    BSbic <- sort(as.double( unlist(strsplit(row['PFbic'], ",")) ), decreasing=T)   # sorted list of BSs
    BSbic <- BSbic[1:floor(length(BSbic)*proportion)]       
    BSnone <- sort(as.double( unlist(strsplit(row['PFnone'], ",")) ), decreasing=T)
    BSnone <- BSnone[1:floor(length(BSnone)*proportion)]  
    p <- wilcox.test(BSbic, BSnone, alternative = "two.sided")$p.value
    return(p)
  }))
  
  print("DATASETS WITH SIGNIFICANTLY (p < 0.05) CHANGED DISTRIBUTION OF BSs COMPARED TO THE BIC TREE: ")
  print( "bic - aicc:")
  print( df.mwbs[which(df.mwbs$Pbs.bic_v_aicc<0.05), 'dataset'])
  print( "bic - all:")
  print( df.mwbs[which(df.mwbs$Pbs.bic_v_all<0.05), 'dataset'])
  print( "bic - none:")
  print( df.mwbs[which(df.mwbs$Pbs.bic_v_none<0.05), 'dataset'])
  print("=============================================================================")
  

  
  return(df.mwbl)
}

significance_BLBS <- function(df, proportion)
{
  
  
  # Mann Whitney test of differences between distribution of Branchlengths under different schemes
  
  
  
  
  # Mann Whitney test of differences between distribution of BS under different schemes
  df.bs_lists <- unstack( data.frame(df$BS.all, df$dataset) )
  #rownames(df.bs_lists) <- c("BSaicc", "BSall", "BSbic", "BSnone")
  rownames(df.bs_lists) <- c("BSbic", "BSaicc", "BSall", "BSnone")
  df.bs_lists <- t(df.bs_lists)
  df.mwbs <- data.frame( Pbs.bic_v_aicc=apply(df.bs_lists, 1, function(row){
    BSbic <- as.double( unlist(strsplit(row[1], ",")) )
    BSaicc <- as.double( unlist(strsplit(row[2], ",")) )
    p <- wilcox.test(BSbic, BSaicc)$p.value 
    return(p)    
  }))
  df.mwbs <- cbind( df.mwbs, Pbs.bic_v_all=apply(df.bs_lists, 1, function(row){
    BSbic <- as.double( unlist(strsplit(row[1], ",")) )
    BSall <- as.double( unlist(strsplit(row[3], ",")) )
    p <- wilcox.test(BSbic, BSall)$p.value
    return(p)    
  }))
  df.mwbs <- cbind( df.mwbs, Pbs.bic_v_none=apply(df.bs_lists, 1, function(row){
    BSbic <- as.double( unlist(strsplit(row[1], ",")) )
    BSnone <- as.double( unlist(strsplit(row[4], ",")) )
    p <- wilcox.test(BSbic, BSnone)$p.value
    return(p)    
  }))
  
  df2 <- cbind(dataset=rownames(df.bl), df.bl, df.bs, df.mwbl, df.mwbs)
  
  return(df2)
}

# 4. Examine the conflicted and congruent nodes
analysis_conflicted_nodes <- function(df)
{
  df.bl <- extract_BLs(df)
  df.bs <- extract_BSs(df)
  
  print("TOTAL # INTERNAL NODES ACROSS ALL DATASETS - NOT INCLUDING PFBIC")
  print("======================================================")
  print(sum(df[df$PF.scheme!='PFbic','nodes.internal']))
        
  print("TOTAL # CONFLICTED NODES BY PSCHEME")
  print("======================================================")
  print( table( subset(df.bl, subset=type=='diff')$PF.scheme) )
  
  print("MEAN BL OF CONFLICTED NODES BY PSCHEME")
  print("======================================================")
  print(aggregate(df.bl, by=list(df.bl$type, df.bl$PF.scheme), FUN=mean))
  
  print("MEAN BS OF CONFLICTED NODES BY PSCHEME")
  print("======================================================")
  print(aggregate(df.bs, by=list(df.bs$type, df.bs$PF.scheme), FUN=mean))
  
  #    - compare the difference between the distrib of BLs of Diffs and Sames
  fig3a <- ggplot(data = df.bl[df.bl$PF.scheme!='PFbic',], aes(x = PF.scheme, y = log(bl))) + 
    geom_boxplot(aes(fill = type), width = 0.6, position = position_dodge(width=0.6)) +
    scale_fill_manual(name = "node type", values = c("white", "dark grey")) + 
    scale_x_discrete("", labels=c("Opt\nAICc", "Adhoc\nmax", "Adhoc\nnone")) + 
    scale_y_continuous("log(branch length)") +
    theme(axis.text=element_text(size=10), axis.title=element_text(size=10))
  #    - compare the difference between the distrib of BSs of Diffs and Sames
  fig3b <- ggplot(data = df.bs[df.bs$PF.scheme!='PFbic',], aes(x = PF.scheme, y = bs)) + 
    geom_boxplot(aes(fill = type), width = 0.6, position = position_dodge(width=0.6)) +
    scale_fill_manual(name = "node type", values = c("white", "dark grey")) + 
    scale_x_discrete("", labels=c("Opt\nAICc", "Adhoc\nmax", "Adhoc\nnone")) + 
    scale_y_continuous("bootstrap support") +
    theme(axis.text=element_text(size=10), axis.title=element_text(size=10))
  grid.arrange(fig3a, fig3b, ncol=1)
  
  print("SIGNIFICANCE OF DIFFERENCE IN BL AND BS BETWEEN CONFLICTS AND CONGRUENTS")
  print("========================================================================")
  #    - significance test of BL values of Diffs and Sames nodes
  print( wilcox.test(df.bl[df.bl$type == "diff", 1], df.bl[df.bl$type == "same", 1], alternative="two.sided") )
  #    - significance test of BS values of Diffs and Sames nodes
  print( wilcox.test(df.bs[df.bs$type == "diff", 1], df.bs[df.bs$type == "same", 1], alternative="two.sided") )
  
  print("SIGNIFICANCE OF DIFFERENCE IN CONFLICTED NODES BL BY SCHEME")
  print("=============================================================================")
  print(wilcox.test( (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFnone'),1]), (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFall'),1]), alternative="two.sided"))
  print(wilcox.test( (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFnone'),1]), (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFaicc'),1]), alternative="two.sided"))
  print(wilcox.test( (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFaicc'),1]), (df.bl[(df.bl$type=='diff' & df.bl$PF.scheme=='PFall'),1]), alternative="two.sided"))

  
  print("SIGNIFICANCE OF DIFFERENCE IN CONFLICTED NODES BS BY SCHEME")
  print("=============================================================================")
  print(wilcox.test( (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFnone'),1]), (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFall'),1]), alternative="two.sided"))
  print(wilcox.test( (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFnone'),1]), (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFaicc'),1]), alternative="two.sided"))
  print(wilcox.test( (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFaicc'),1]), (df.bs[(df.bs$type=='diff' & df.bs$PF.scheme=='PFall'),1]), alternative="two.sided"))
  
  
}

analysis_BL_depth_slope <- function(df)
{
  slope.bic <- subset(df, subset = PF.scheme=='PFbic', slope.BLdepth)
  slope.none <- subset(df, subset = PF.scheme=='PFnone', slope.BLdepth)
  return(wilcox.test(slope.bic$slope.BLdepth, slope.none$slope.BLdepth, alternative = "two.sided", paired=TRUE))
}



############ Helper functions #########################



# compare distribution of BL and BS per dataset due to partitioning scheme choice
# this code is really ugly but i couldn't be stuffed thinking of a more 'R'-ish way to do it.
# -- proportion refers to the top X values to use in the distribution MW test. (e.g. 0.25 for top quarter. 1.0 for all)


extract_list <- function(df, scheme, variable) {
  d <- subset(df, subset=PF.scheme==scheme, select=variable)
  d <- unlist( strsplit(d[,variable], ",") )
  d <- as.double(d)
  return(d)
}

extract_BLs <- function(df)
{
  # get a combined dataframe of BL values (by sames, diffs and PFscheme)  in long format
  d.1 <- extract_list(df, 'PFbic', 'Diffs.BL')
  d.2 <- extract_list(df, 'PFbic', 'Sames.BL')
  d.3 <- extract_list(df, 'PFaicc', 'Diffs.BL')
  d.4 <- extract_list(df, 'PFaicc', 'Sames.BL')
  d.5 <- extract_list(df, 'PFall', 'Diffs.BL')
  d.6 <- extract_list(df, 'PFall', 'Sames.BL')
  d.7 <- extract_list(df, 'PFnone', 'Diffs.BL')
  d.8 <- extract_list(df, 'PFnone', 'Sames.BL')
  d.bl <- data.frame( bl=c(d.1, d.2, d.3, d.4, d.5, d.6, d.7, d.8), 
                      type=c(rep('diff', times=length(d.1)), rep('same', times=length(d.2)), rep('diff', times=length(d.3)), rep('same', times=length(d.4)), rep('diff', times=length(d.5)), rep('same', times=length(d.6)), rep('diff', times=length(d.7)), rep('same', times=length(d.8))), 
                      PF.scheme=c(rep('PFbic', times=length(d.1)+length(d.2)), rep('PFaicc', times=length(d.3)+length(d.4)), rep('PFall', times=length(d.5)+length(d.6)), rep('PFnone', times=length(d.7)+length(d.8))) )
  return(d.bl)
}

#ggplot(BL[BL$PF.scheme!='PFbic',], aes(x=PF.scheme, y=log10(bl), fill=type)) + geom_boxplot(outlier.colour="azure4", col='bisque4', width=0.5, , position=position_dodge(width=0.9))

extract_BSs <- function(df)
{
  # get a combined dataframe of  BS values (by sames, diffs and PFscheme)  in long format
  d.1 <- extract_list(df, 'PFbic', 'Diffs.BS')
  d.2 <- extract_list(df, 'PFbic', 'Sames.BS')
  d.3 <- extract_list(df, 'PFaicc', 'Diffs.BS')
  d.4 <- extract_list(df, 'PFaicc', 'Sames.BS')
  d.5 <- extract_list(df, 'PFall', 'Diffs.BS')
  d.6 <- extract_list(df, 'PFall', 'Sames.BS')
  d.7 <- extract_list(df, 'PFnone', 'Diffs.BS')
  d.8 <- extract_list(df, 'PFnone', 'Sames.BS')
  d.bs <- data.frame( bs=c(d.1, d.2, d.3, d.4, d.5, d.6, d.7, d.8), 
                      type=c(rep('diff', times=length(d.1)), rep('same', times=length(d.2)), rep('diff', times=length(d.3)), rep('same', times=length(d.4)), rep('diff', times=length(d.5)), rep('same', times=length(d.6)), rep('diff', times=length(d.7)), rep('same', times=length(d.8))), 
                      PF.scheme=c(rep('PFbic', times=length(d.1)+length(d.2)), rep('PFaicc', times=length(d.3)+length(d.4)), rep('PFall', times=length(d.5)+length(d.6)), rep('PFnone', times=length(d.7)+length(d.8))) )
  return(d.bs)
}

#ggplot(d.bs[d.bs$PF.scheme!='PFbic',], aes(x=PF.scheme, y=bs, fill=type)) + geom_hline(yintercept=95, linetype='dashed', colour="grey", alpha=0.8) + geom_boxplot(outlier.colour="azure4", col='bisque4', width=0.5, , position=position_dodge(width=0.9))

show_distribution <- function(df, scheme, variable, xmax=100, ymax=100, binwidth=5 ){
  
  d <- extract_list(df, scheme, variable)
  # plot it
  qplot(d, geom="histogram", binwidth=binwidth, xlim=c(0,xmax), ylim=c(0,ymax), main=paste('BIC vs ', scheme, ' - conflicted branches', sep=''))   
}



######################################################################################################   
#####  read in the datasets summary csv and merge with Coredata                  #####################
######################################################################################################  

dnafactor <- function(mito, nuc, chlor)
{
  dnatypes = character()
  if (mito=="TRUE") dnatypes <- c(dnatypes, 'M')
  if (nuc=="TRUE") dnatypes <- c(dnatypes, 'N')
  if (chlor=="TRUE") dnatypes <- c(dnatypes, 'C')
  return( paste(dnatypes, collapse=""))
}

getMetadata <- function(df)
{
  metadata <- read.csv( paste(basepath, "/metadata_summary.csv",sep=""), header=TRUE)
  names(metadata)[1] <- "dataset"
  df <- merge(df, metadata, by="dataset")
  
  df$dnatype <- ''
  df$dnatype <- mapply( dnafactor, df$mitochondrial, df$nuclear, df$chloroplast )
  df$dnatype <- as.factor(df$dnatype)
  return(df)
}


######################################################################################################   
#####  read in the partitioning and tree inference score data (lnL, aicc, bic values) to evaluate ####
######################################################################################################  
getScoredata <- function(df)
{
  scoredata <- read.csv( paste(basepath, "/scoredata.csv",sep=""), header=TRUE)
  scoredata <- scoredata[scoredata$dataset%in%(datasets),-c(11:14)]
  scoredata.lnL <- subset(scoredata, scoredata$Measure=="lnL", select=c(1:10))
  scoredata.aicc <- subset(scoredata, scoredata$Measure=="aicc", select=c(1:10))
  scoredata.bic <- subset(scoredata, scoredata$Measure=="bic", select=c(1:10))
  df <- merge(df, scoredata.bic, by="dataset")
  return(df)
}


plot_12panel_figure <- function(dframe)
{
    df1 <<- dframe
    
  fig1 <- ggplot(df1, aes(x=sites/datablocks, y=pmin(PFbic_GTRG, PFbic_GTRIG)-pmin(PFall_GTRG, PFall_GTRIG))) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-30000,1000), name=expression(paste(Delta,"BIC"))) +
    ggtitle(expression(Adhoc[feat])) + 
    theme(axis.text=element_text(size=10), axis.title=element_text(size=10))
  
  fig2 <- ggplot(df1, aes(x=sites/datablocks, y=0)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-30000,1000), labels=NULL, name="") + 
    ggtitle(expression(Opt[AICc])) +
    theme(axis.text=element_text(size=10), axis.title=element_text(size=10))
  
  fig3 <- ggplot(df1, aes(x=sites/datablocks, y=pmin(PFbic_GTRG, PFbic_GTRIG)-pmin(PFnone_GTRG, PFnone_GTRIG) )) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-30000,1000), labels=NULL, name="")  + 
    ggtitle(expression(Adhoc[one])) +
    theme(axis.text=element_text(size=10), axis.title=element_text(size=10))
  
  fig4 <- ggplot(df1, aes(x=sites/datablocks, y=bl.bic_v_all)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-0.05,0.05), name=expression(paste(Delta,"BL")))
  
  fig5 <- ggplot(df1, aes(x=sites/datablocks, y=bl.bic_v_aicc)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-0.05,0.05),labels=NULL, name="")
  
  fig6 <- ggplot(df1, aes(x=sites/datablocks, y=bl.bic_v_none)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-0.05,0.05),labels=NULL, name="")
  
  fig7 <- ggplot(df1, aes(x=sites/datablocks, y=bs.bic_v_all)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-5,5), name=expression(paste(Delta,"BS")))
  
  fig8 <- ggplot(df1, aes(x=sites/datablocks, y=bs.bic_v_aicc)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-5,5),labels=NULL, name="")
  
  fig9 <- ggplot(df1, aes(x=sites/datablocks, y=bs.bic_v_none)) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900), labels=NULL, name="") + 
    scale_y_continuous(limits=c(-5,5),labels=NULL, name="")
  
  fig10 <- ggplot(df1[df1$PF.scheme=='PFall',], aes(x=sites/datablocks, y=df1[df1$PF.scheme=='PFall', 3])) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900)) + 
    scale_y_continuous(limits=c(0,0.4), name="MS distance")
  
  fig11 <- ggplot(df1[df1$PF.scheme=='PFaicc',], aes(x=sites/datablocks, y=df1[df1$PF.scheme=='PFaicc', 3])) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900)) + 
    scale_y_continuous(limits=c(0,0.4),labels=NULL, name="")
  
  fig12 <- ggplot(df1[df1$PF.scheme=='PFnone',], aes(x=sites/datablocks, y=df1[df1$PF.scheme=='PFnone', 3])) + 
    geom_point(size=1.2) + 
    scale_x_continuous(limits=c(100,900)) + 
    scale_y_continuous(limits=c(0,0.4),labels=NULL, name="")


  grid.arrange(fig1,fig2,fig3,fig4,fig5,fig6,fig7,fig8,fig9,fig10,fig11,fig12)
  return()
}

# see if the various measures (topo distance, deltaBL, deltaBS) are affected by the size of the alignment (sites)
analysis_bysites <- function(dframe)
{
  df1 <<- dframe
  
  # remove the rows for PFbic since we are only looking at PFbic vs PFnone, PFaicc and PFall.
  df1 <- subset(df1, subset=PF.scheme!='PFbic')
  
  # Changes in Topology as a function of sites
  fig1 <- ggplot(df1) + geom_point( aes(x=sites, y=dist.MS, color=PF.scheme, shape=PF.scheme), size=2 ) + 
          scale_color_discrete(name="OptBIC vs") + scale_shape_discrete(name="OptBIC vs") + 
          scale_y_continuous(name="MS distance") +
          theme(legend.position=c(1,1), legend.justification=c(1,1))
  fig2 <- ggplot(df1) + geom_point( aes(x=sites, y=nodes.numdiff, color=PF.scheme, shape=PF.scheme), size=2 ) + 
          scale_y_continuous(name="conflicted nodes") +
          theme(legend.position="none")
  
  # Changes in BL as a function of sites
  melted <- melt(data = df1, id.vars = c('dataset', 'sites'), measure.vars = c('bl.bic_v_aicc', 'bl.bic_v_all', 'bl.bic_v_none'), variable.name='vars', value.name='deltaBL')
  fig3 <- ggplot(melted) + geom_point( aes(x=sites, y=deltaBL, color=vars, shape=vars), size=2 )  + 
          scale_y_continuous(name=expression(paste(Delta, "BL"))) +
          theme(legend.position="none")
    
  # Changes in BS as a function of sites
  melted <- melt(data = df1, id.vars = c('dataset', 'sites'), measure.vars = c('bs.bic_v_aicc', 'bs.bic_v_all', 'bs.bic_v_none'), variable.name='vars', value.name='deltaBS')
  fig4 <- ggplot(melted) + geom_point( aes(x=sites, y=deltaBS, color=vars, shape=vars), size=2 )  + 
          scale_y_continuous(name=expression(paste(Delta, "BS"))) +
          theme(legend.position="none")
  grid.arrange(fig1,fig2,fig3,fig4)
  return()
}


###########   analysis    ########################

# use all datasets
core.bak  <- core
core      <- getScoredata(core)
core      <- getMetadata(core)
analysis_topo_distances(core)
analysis_numconflicts(core)
analysis_meanBLBS(core, 1.0)

core <- merge(core, df.ptested, by="dataset")
analysis_conflicted_nodes(core)
plot_12panel_figure(core)
analysis_bysites(core)
analysis_BL_depth_slope(core)

# outliers removed - re analyse
core                      <- core.bak
core                      <- getScoredata(core)
core                      <- getMetadata(core)
outliers                  <- unique( core$dataset[which(core$BL.mean > 0.15)] )
core.no_outliers          <- core[!core$dataset%in%outliers,]
rownames(core.no_outliers) <- NULL
# core.no_outliers$dataset  <- factor(core.no_outliers$dataset)
analysis_topo_distances(core.no_outliers)
analysis_numconflicts(core.no_outliers)
analysis_meanBLBS(core.no_outliers, 1.0)
core.no_outliers          <- merge(core.no_outliers, df.ptested, by="dataset")

analysis_conflicted_nodes(core.no_outliers)
plot_12panel_figure(core.no_outliers)
analysis_bysites(core.no_outliers)
analysis_BL_depth_slope(core.no_outliers)


### plot a relationship between two vars, e.g. BL.mean and BS.mean, GROUPED by PF scheme
# plot(outputdata$Diffs.PFnone.BL.mean ~ outputdata$Diffs.PFnone.BS.mean, log="xy", col=c("red","blue","orange"), pch=16)
# legend("bottomright",pfschemes[1:3], fill=c("red","orange","blue"))




### extract a character list, such as BL.all, and get it into numeric (double) vector format
# BL.all <- outputdata[1,"BL.all"]
# BL <- unlist( strsplit(BL.all, ",") )
# BL <- as.double(BL)

### get the highest N values per cell from a column containing comma delimited lists
# BS.bic.highest <- lapply( BS.bic$Diffs.PFversus.BS, FUN=function(x){ tail(sort(as.numeric(unlist(strsplit(x,",")))), 5) } )

######### useful R phylo stuff
# 1. plot a tree showing Node IDs
#   tree$node.label <- NULL   # do this to get rid of BS values which override the node IDs as labels
#   plot(tree, cex=0.6)
#   nodelabels(bg="transparent",frame="none", cex=0.6, adj=-0.5)

# OR
# plot.phylo(fong.bic, cex=0.5, label.offset=0.01, no.margin=T)
# nodelabels(bg="transparent",frame="none", cex=0.5, adj=-0.1, col="red")

# 2. plot two trees side by side showing the different branches
#   phylo.diff(tree1, tree2)

# 3. DISTANCE between trees
# Can use dist.topo {ape}, treedist {phangorn}, dist.multiphylo {distory}
#   treedist's branch.score.difference value appears to be very similar to dist.multiphylo geodesic values
#   NOTE:: dist.multiPhylo doesn't work when node labels are present, so remove them first

# 4. COMPARING trees:
# prop.clades - gets the list of nodes that are different between a tree and 1+ other trees  (ape package)
